@@ -602,6 +602,11 @@ class Object(Client):
         }.items() if value}
         return self.client.request('DELETE', self.uri, **headers)  # type: ignore # <https://github.com/python/mypy/issues/10008>
 
+    def restore(self, reload: bool = True) -> Response:
+        """Restore an object from the recyclebin."""
+        uri = self.client.get_relation(self.hal, 'udm:restore')['href']
+        return self._request('POST', uri, {}, {}, reload=reload)
+
     def move(self, position: str, reload: bool = True) -> None:
         self.position = position
         self.save(reload=reload)
